/** A double-ended queue. All queue operations are `O(1)`. */
export declare class Queue<T> {
    #private;
    /** Returns the capacity of the queue. That is, that of the inner buffer. */
    get capacity(): number;
    /** Returns the current number of elements in the queue. */
    get length(): number;
    /** Returns whether the deque is empty. */
    get empty(): boolean;
    /** Performs a "soft" clear. This does **not** reset capacity. */
    clear(): void;
    /** Inserts item to first slot. Returns the new length of the deque. */
    unshift(item: T): number;
    /** Removes and returns the first element. */
    shift(): T;
    /** Removes and returns the first element or undefined.  */
    shiftMaybe(): T | undefined;
    expect(expected?: T): T;
    /** Inserts item to the last slot. Returns the new length of the deque. */
    push(item: T): number;
    /** Removes and returns the last element. */
    pop(): T | undefined;
    /** View the item at the specific index (without removing). */
    at(index: number): T | undefined;
    [Symbol.iterator](): Generator<T | undefined, void, unknown>;
    private shrinkArray;
    private growArray;
}
